@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2025  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -42,6 +42,7 @@
 
 #ifdef NMZ_FLINT
 #include "flint/flint.h"
+#include "flint/fmpz.h"
 #include "flint/fmpz_poly.h"
 #endif
 
@@ -72,7 +73,10 @@ void flint_poly(fmpz_poly_t flp, const vector<mpz_class>& nmzp) {
     slong n = (slong)nmzp.size();
     fmpz_poly_fit_length(flp, n);
     for (size_t i = 0; i < nmzp.size(); ++i) {
-        fmpz_poly_set_coeff_mpz(flp, (slong)i, nmzp[i].get_mpz_t());
+        fmpz_t fc;
+        fmpz_init(fc);
+        fmpz_set_mpz(fc, nmzp[i].get_mpz_t());
+        fmpz_poly_set_coeff_fmpz(flp, (slong)i, fc);
     }
 }
 
@@ -80,9 +84,12 @@ void nmz_poly(vector<mpz_class>& nmzp, const fmpz_poly_t flp) {
     size_t n = (size_t)fmpz_poly_length(flp);
     nmzp.resize(n);
     mpz_t c;
+    fmpz_t fc;
     mpz_init(c);
+    fmpz_init(fc);
     for (size_t i = 0; i < nmzp.size(); ++i) {
-        fmpz_poly_get_coeff_mpz(c, flp, i);
+        fmpz_poly_get_coeff_fmpz(fc, flp, i);
+        fmpz_get_mpz(c, fc);
         nmzp[i] = mpz_class(c);
     }
     mpz_clear(c);
@@ -294,6 +301,8 @@ void HilbertSeries::initialize() {
     nr_coeff_quasipol = -1;  // all coefficients
     expansion_degree = -1;
     period_bounded = true;
+    only_cyclotomic = false;
+    allow_quasipoly = true;
 }
 
 // Constructor, creates 0/1
@@ -367,6 +376,32 @@ void HilbertSeries::set_period_bounded(bool on_off) const {  // period_bounded i
 bool HilbertSeries::get_period_bounded() const {
     return period_bounded;
 }
+
+void HilbertSeries::set_only_cyclotomic(bool on_off) const{
+    only_cyclotomic = on_off;
+}
+
+bool HilbertSeries::get_only_cyclotomic() const{
+    return only_cyclotomic;
+}
+
+void HilbertSeries::forbid_quasipol(bool on_off) const{
+    allow_quasipoly = !on_off;
+}
+
+bool HilbertSeries::get_quasipol_allowed() const{
+    return allow_quasipoly;
+}
+
+void HilbertSeries::get_variants(const HilbertSeries& mother){
+    set_expansion_degree(mother.get_expansion_degree());
+    set_nr_coeff_quasipol(mother.get_nr_coeff_quasipol());
+    forbid_quasipol(!mother.get_quasipol_allowed());
+    set_only_cyclotomic(mother.get_only_cyclotomic());
+    set_period_bounded(mother.get_period_bounded());
+    //HSOP is extra
+}
+
 // add another HilbertSeries to this
 void HilbertSeries::add(const vector<num_t>& num, const vector<denom_t>& gen_degrees) {
     vector<denom_t> sorted_gd(gen_degrees);
@@ -450,12 +485,31 @@ void HilbertSeries::collectData() const {
 
 // simplify, see class description
 void HilbertSeries::simplify() const {
+
     if (is_simplified)
         return;
     collectData();
-    /*    if (verbose) {
+
+    size_t highest_comp = num.size();
+    bool non_zero = false;
+    // First we remove "leading" 0 from the numerator
+    for(size_t i = 0; i < num.size(); ++i){
+        if(num[i] != 0){
+            highest_comp = i;
+            non_zero = true;
+        }
+    }
+    if(non_zero)
+        num.resize(highest_comp + 1);
+    else
+        num.clear();
+
+    computeDegreeAsRationalFunction();
+
+    /*if (verbose) {
             verboseOutput() << "Hilbert series before simplification: "<< endl << *this;
-        }*/
+        }
+    */
     vector<mpz_class> q, r, poly;  // polynomials
     // In denom_cyclo we collect cyclotomic polynomials in the denominator.
     // During this method the Hilbert series is given by num/(denom*cdenom)
@@ -483,7 +537,6 @@ void HilbertSeries::simplify() const {
         }
         if (denom_i == 0)
             continue;
-
         // decompose (1-t^i) into cyclotomic polynomial
         for (long d = 1; d <= i / 2; ++d) {
             if (i % d == 0)
@@ -494,6 +547,7 @@ void HilbertSeries::simplify() const {
         if (denom_i % 2 == 1)
             v_scalar_multiplication(num, mpz_class(-1));
     }  // end for
+
     denom.clear();
 
     auto it = cdenom.begin();
@@ -528,13 +582,23 @@ void HilbertSeries::simplify() const {
     cyclo_num = num;
     cyclo_denom = cdenom;
 
+    period = lcm_of_keys(cdenom);
+
+    if(only_cyclotomic){
+        expansion_degree = -1;
+        nr_coeff_quasipol = 0;
+        num.clear();
+        denom.clear();
+        is_simplified = true;
+        return;
+    }
+
     // now collect the cyclotomic polynomials in (1-t^i) factors
     it = cdenom.find(1);
     if (it != cdenom.end())
-        dim = it->second;
+        dim = it->second; // dim = number of occurrences of 1
     else
         dim = 0;
-    period = lcm_of_keys(cdenom);
     if (period_bounded && period > 10 * PERIOD_BOUND) {
         if (verbose) {
             errorOutput() << "WARNING: Period is too big, the representation of the Hilbert series may have more than dimension "
@@ -584,12 +648,11 @@ void HilbertSeries::simplify() const {
         }
     }
     is_simplified = true;
-    computeDegreeAsRationalFunction();
     quasi_poly.clear();
 }
 
 void HilbertSeries::computeDegreeAsRationalFunction() const {
-    simplify();
+    // simplify();
     long num_deg = num.size() - 1 + shift;
     long denom_deg = 0;
     for (auto& it : denom) {
@@ -631,7 +694,7 @@ mpz_class HilbertSeries::getHilbertQuasiPolynomialDenom() const {
 }
 
 void HilbertSeries::computeHilbertQuasiPolynomial() const {
-    if (isHilbertQuasiPolynomialComputed() || nr_coeff_quasipol == 0)
+    if (isHilbertQuasiPolynomialComputed() || !allow_quasipoly)
         return;
     simplify();
 
@@ -882,7 +945,6 @@ void HilbertSeries::adjustShift() {
             denom[d] = 1;
  }
 
-/*
 // methods for textual transfer of a Hilbert Series
 string HilbertSeries::to_string_rep() const {
     collectData();
@@ -947,7 +1009,7 @@ ostream& operator<<(ostream& out, const HilbertSeries& HS) {
     out << " )" << std::endl;
     return out;
 }
-*/
+
 
 //---------------------------------------------------------------------------
 // polynomial operations, for polynomials repr. as vector of coefficients
