@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2025  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -144,7 +144,7 @@ Matrix<Integer> readMatrix(const string project) {
     ifstream in;
     in.open(file_in, ifstream::in);
     if (in.is_open() == false)
-        throw BadInputException("readMatrix cannot find file");
+        throw BadInputException("readMatrix cannot find file " + project);
     int nrows, ncols;
     in >> nrows;
     in >> ncols;
@@ -441,6 +441,7 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
         current_term += test;
     }
     terms.push_back(current_term);
+    // cout << terms;
     if (!relation_read)
         throw BadInputException("No relation in constraint");
 
@@ -533,12 +534,16 @@ void read_symbolic_constraint(istream& in, string& rel, vector<Number>& left, Nu
         }
 
         if (comp_string != "") {
+
+            // cout << "CCCCCCCCCC " << comp_string << endl;
             bool bracket_read = false;
             string expo_string;
             for (char j : comp_string) {
                 if (j == ']')
                     break;
                 if (j == '[') {
+                    if(bracket_read)
+                        throw BadInputException("Double [ in term.");
                     bracket_read = true;
                     continue;
                 }
@@ -719,8 +724,10 @@ void read_polynomial_constraints(istream& in, vector<string>& polynomial_constra
 
     int nr_constraints;
     in >> nr_constraints;
-    if(in.fail() || nr_constraints<= 0)
+    if(in.fail() || nr_constraints< 0)
         throw BadInputException("Failure in reading number of polynomial constraints!");
+    if(nr_constraints == 0)
+        return;
 
     string equ;
     for(int i = 0; i < nr_constraints; ++i){
@@ -903,6 +910,9 @@ InputMap<Number> readNormalizInput(istream& in,
     set<NumParam::Param> num_par_already_set;
     bool we_have_a_polynomial = false;
     bool convert_equations = false;
+    no_coord_transf = false; // in general.h and cpp
+
+    write_lp_file = false; // in general.h and cpp
     size_t length_weight = 0;
     bool monoid_read = false;
     bool lattice_ideal_read = false;
@@ -1014,8 +1024,8 @@ InputMap<Number> readNormalizInput(istream& in,
                     options.activateNoMatricesOutput();
                     continue;
                 }
-                if (type_string == "NoOutputOnInterrupt") {
-                    options.activateNoOutputOnInterrupt();
+                if (type_string == "OutputOnInterrupt") {
+                    options.activateOutputOnInterrupt();
                     continue;
                 }
                 if (type_string == "NoSuppHypsOutput") {
@@ -1072,6 +1082,18 @@ InputMap<Number> readNormalizInput(istream& in,
                     convert_equations = true;
                     continue;
                 }
+                if (type_string == "no_coord_transf") {
+                    no_coord_transf = true;
+                    continue;
+                }
+                if (type_string == "write_lp_file") {
+                    write_lp_file = true;
+                    continue;
+                }
+                if (type_string == "list_polynomials") {
+                    polynomial_verbose = true;
+                    continue;
+                }
                 if(type_string == "parallel_threads"){
 
                     long nr_threads;
@@ -1081,8 +1103,35 @@ InputMap<Number> readNormalizInput(istream& in,
                     set_thread_limit(nr_threads);
                     continue;
                 }
-
+                
                 input_type = to_type(type_string);
+                
+                if(type_string == "fusion_image_type" || type_string == "fusion_image_duality"
+                    || type_string == "fusion_image_ring"){
+                    
+                    in >> std::ws;  // eat up any leading white spaces
+                    c = in.peek();
+                    if(c != '[')
+                        throw BadInputException("Fusion image data must be formatted vectors");
+                    vector<Number> input_vec;
+                    read_formatted_vector(in, input_vec);
+                    save_matrix(input_map, input_type, Matrix<Number>(input_vec));
+                    continue;
+                }
+                
+                if(type_string == "fusion_ring_map"){
+                    
+                    in >> std::ws;  // eat up any leading white spaces
+                    c = in.peek();
+                    if(c != '[')
+                        throw BadInputException("Fusion ring map musat be formatted matrix");
+                
+                    Matrix<Number> input_mat;
+                    read_formatted_matrix(in, input_mat, false);
+                    save_matrix(input_map, input_type, input_mat);
+                    continue;
+                }
+
 
                 if(type_string == "monoid")
                     monoid_read = true;
@@ -1132,8 +1181,8 @@ InputMap<Number> readNormalizInput(istream& in,
 
                         Matrix<Number> e_i = Matrix<Number>(vector<Number>(nr_columns, 0));
                         if (pos < 1 || pos > static_cast<long>(e_i[0].size())) {
-                            throw BadInputException("Error while reading " + type_string + " as a unit_vector " + toString(pos) +
-                                                    "!");
+                            throw BadInputException("Error while reading " + type_string + " as a unit_vector " + toString(pos)
+                             +    "!");
                         }
                         pos--;  // in input file counting starts from 1
                         e_i[0].at(pos) = 1;
